@REM #########################################################################
@REM # IBM Confidential
@REM # OCO Source Materials
@REM # 5724-S51
@REM #
@REM # (C) Copyright IBM Corp. 2008  All Rights Reserved.
@REM #
@REM # The source code for this program is not published or otherwise
@REM # divested of its trade secrets, irrespective of what has been
@REM # deposited with the U.S. Copyright Office.
@REM #########################################################################
@echo off
setlocal ENABLEEXTENSIONS

REM ==========================================================================
REM  Set up local constants
REM ==========================================================================
set PATH=%SystemRoot%\System32;%SystemRoot%;%SystemRoot%\system32\wbem;%PATH%
set LOGFILE=%temp%\IBM_Tivoli_Netcool_Webtop_InterimFix.log
set ERROR=0
set DEFAULT_INSTALLATION_DIR=C:\IBM\tivoli\tip

pushd %~dp0
set HERE=%CD%
popd

echo IBM Tivoli Netcool/Webtop 2.2 InterimFix 003

REM ==========================================================================
REM  Parse the command line arguments
REM ==========================================================================
:PROCESSARGS
   if "%1" == ""   goto :CHECKARGS
   if "%1" == "-i" set DO=update
   if "%1" == "-r" set DO=undo

:CHECKARGS
   if "%DO%" == "" (
      call :USAGE
      exit /b 1
   )

REM ==========================================================================
REM  Get the location of the TIP installation on this system
REM ==========================================================================
:GETTIP
   if EXIST "%TIP_INSTALL%" goto :GOTTIP
   set /p TIP_INSTALL=Please enter path to Tivoli Integrated Portal installation directory [%DEFAULT_INSTALLATION_DIR%]
   if "%TIP_INSTALL%" == "" set TIP_INSTALL=%DEFAULT_INSTALLATION_DIR%
   goto :GETTIP

:GOTTIP
   pushd %TIP_INSTALL%
   set TIP_INSTALL=%cd%
   popd

REM ==========================================================================
REM  Set up the DE environment
REM ==========================================================================
   echo Setting up Deployment Engine environment.
   if EXIST "C:\Program Files\IBM\Common\acsi\setenv.cmd" (
      call "C:\Program Files\IBM\Common\acsi\setenv.cmd"
   ) else (
      call "C:\Program Files (x86)\IBM\Common\acsi\setenv.cmd"
   )
   @echo off
   if "%DO%" == "update" (
      echo Installing Deployment Engine...
      call "%~dp0DE\si_inst.bat" 1>>"%LOGFILE%" 2>&1
   )

REM ==========================================================================
REM  Process the patches as requested by the user on the command line
REM ==========================================================================
   for /f "tokens=1*" %%i in ('sc query state^= all ^| find "IBMWAS61Service"') do (
      for /f "tokens=*" %%k in ('sc qc "%%j" ^| find "%TIP_INSTALL%\bin\wasservice.exe"') do (
        sc query "%%j" | find "RUNNING" 1>nul 2>&1
         if NOT ERRORLEVEL 1 (
            if "%DO%" == "update" (
               echo Please stop Tivoli Integrated Portal before installing InterimFix.
            ) else (
               echo Please stop Tivoli Integrated Portal before uninstalling InterimFix.
            )
            goto :END
         )
      )
   )

   call :ProcessFix %DO% "%TIP_INSTALL%" CommonReporting-TIP-1200-CommonReporting-0001
   call :ProcessFix %DO% "%TIP_INSTALL%" ISC-TIP-7500-ISC-0003
   call :ProcessFix %DO% "%TIP_INSTALL%" ISC-TIP-7500-ISC-0004
   REM call :ProcessFix %DO% "%TIP_INSTALL%" ITMWebService-TIP-1100-ITMWebService-0002
   call :ProcessFix %DO% "%TIP_INSTALL%" TIP-TIP-1100-TIP-0002
   REM call :ProcessFix %DO% "%TIP_INSTALL%" TIPChart-TIP-1100-TIPChart-0002
   call :ProcessFix %DO% "%TIP_INSTALL%" Webtop-22082-InterimFix003

   if %ERROR% == 0 (
      if "%DO%" == "update" (
         echo InterimFix installed successfully.
      ) else (
         echo InterimFix uninstalled successfully.
      )
      echo Please start Tivoli Integrated Portal.
   ) else (
      if "%DO%" == "update" (
         echo InterimFix installation failed.
     ) else (
         echo InterimFix uninstallation failed.
      )
      echo Check details in %LOGFILE%
   )
   goto :END

REM ==========================================================================
REM  Usage Routine
REM ==========================================================================
:USAGE
   echo Usage:  %~n0 -i ^| -r
   echo.
   echo where:
   echo   -i        Install InterimFix.
   echo   -r        Rollback (uninstall) InterimFix.
   echo.
   goto :EOF

REM ==========================================================================
REM  ProcessFix - install a specific fix, holding output to display on error
REM ==========================================================================
:ProcessFix
   pushd %~dp0ifixes\%3\META-INF
   if "%1" == "update" (
      set OPTIONS=-u true
      echo Installing %3...
   ) else (
      echo Uninstalling %3...
   )
   call manageIU.bat -o %1 -r %2 -p packagedIU.xml %OPTIONS% 1>>"%LOGFILE%" 2>&1
   if ERRORLEVEL 1 (
      if "%1" == "update" (
         echo Installation of %3 failed.
      ) else (
         echo Uninstallation of %3 failed.
      )
      set ERROR=3
      endlocal
      exit /b %ERROR%
   )
   popd
   goto :EOF

REM  ==========================================================================
REM   End of Program
REM  ==========================================================================
:END
   endlocal
   exit /b %ERROR%
