/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/
dojo.require( "ibm.tivoli.tcr.dijit.PFieldString" );
dojo.require( "ibm.tivoli.tcr.dijit.PFieldBoolean" );
dojo.require( "ibm.tivoli.tcr.dijit.PFieldDate" );
dojo.require( "ibm.tivoli.tcr.dijit.PFieldDateTime" );
dojo.require( "ibm.tivoli.tcr.dijit.PFieldNumber" );
dojo.require( "ibm.tivoli.tcr.dijit.PFieldSelectionList" );
dojo.require( "ibm.tivoli.tcr.dijit.PFieldTime" );
dojo.require( "ibm.tivoli.tcr.util" );
dojo.require( "ibm.tivoli.tcr.Server" );

dojo.require( "dojo.data.ItemFileReadStore" );

dojo.provide( "ibm.tivoli.tcr.ParameterManager" );
/*******************************************************************************
* ParameterManager
*
* This dojo class is responsible for requesting parameters for a snapshot
* or report defaults to display them for editing.  It will load the parameters,
* build the editor table, and also check to see if there are visible parameters
* available -- to aid in the decision of whether a dialog is even needed for 
* editing.
*
* The manager should be destroyed after use, for it will have lots of Dojo
* objects associated with it.
*******************************************************************************/
dojo.declare( "ibm.tivoli.tcr.ParameterManager", //className
			  null,                              //extends
{
	/***************************************************************************
	* Constructor
	* 
	* REQUIRED PARAMETER ITEMS:
	* global: the tcr global namespace object
	* topic: the topic for the JSON request for parameters
	* context: the context (selected id of report or snapshot) for 
	*          the JSON request of parameters
	***************************************************************************/
	constructor: function( parms )	
	{
      //save all parameters	
      dojo.mixin( this, parms );
      	  
	  //save a reference to all widgets for destruction
	  this.ALL_WIDGETS = [];
	  
	  this.CascadingGroupManagers = [];
	  	  
	  //PARAMETER DATA TYPES
	  this.TYPE_STRING   = 0;	  
	  this.TYPE_FLOAT    = 1;	  
	  this.TYPE_DECIMAL  = 2;	  
	  this.TYPE_BOOLEAN  = 3;	  
	  this.TYPE_INTEGER  = 4;	  
	  this.TYPE_DATETIME = 5;	  
	  this.TYPE_DATE     = 6;	  
	  this.TYPE_TIME     = 7;
    	  
      this.finishedLoading = false;
	  this.loadParameters();	  
	},
	
	/***************************************************************************
	* Load the parameters via AJAX
	***************************************************************************/
	loadParameters: function()
	{
	  kw = { 
	         content:    {
                        topic   : this.topic,
                        command : "getParameters",   
                        context : this.context
       	               },
	         
	         onResponse: { callback: dojo.hitch( this, "_parametersLoaded" ) }
	       };
	  
    ibm.tivoli.tcr.Server.getSingleton( this.global ).get( kw );
	},

  _parametersLoaded: function ( data )
  {
    // if returned data model is empty --> there are no visible parameters
    // otherwise, check visibility of all parameters to see if there is at least
    // one visible parameter
    
    this.containsVisibleParams = ( data.list.length === 0 )
                                 ? false
                                 : this._paramGroupContainsVisibleParams( data.list );

    this.dataModel = data;          
    this.onLoad( this.containsVisibleParams );	
  },	           
  
    /***************************************************************************
    * Recursive function to look for visible parameters within a parameter group
    ***************************************************************************/
    _paramGroupContainsVisibleParams: function( list )
    {
      var containsVisibleParams = false;
  
      for ( var i in list )
      { 
        if ( containsVisibleParams == false)
        {
           if ( list[ i ].type == "2" )
           {
             containsVisibleParams = this._paramGroupContainsVisibleParams( list[i].list );
           }
           else
           {
             if ( list[ i ].hidden === false )
             {
               containsVisibleParams = true;
               break;
             }               
           }
        }
      }
      
      return containsVisibleParams;
    },
  	  
    /***************************************************************************
    * Called when the parameters have finished loading.
    ***************************************************************************/
    onLoad: function( containsVisibleParams ) { /*Attach listeners here */ },
    
    /***************************************************************************
    * Does the manager contain visible parameters?
    ***************************************************************************/
    hasVisibleParams: function()
    {
      return this.containsVisibleParams;
    },

    /***************************************************************************
    * Have the parameters finished loading?
    ***************************************************************************/
    hasFinishedLoading: function()
    {
      return this.finishedLoading;
    },
    
	/*****************************************************************
	* Populate a dijit.form.Form with the parameters as editors.
	*****************************************************************/
	populateForm: function ( form, /* dijit.form.Form */ 
	                         readOnly )
	{	
		//TODO: ENSURE THAT ALL_WIDGETS IS EMPTY -- IF NOT THEN DESTROY THEM
		  
		//create table element and set the features		
		var table = document.createElement('table');
		table.className="param_table";
		table.cellPadding="2px";
		table.cellSpacing="4px";
	
		//place the table on the content pane
		form.domNode.appendChild( table );	

	    //Do not remove this element! IE requires a TBODY element
	    //when building the DOM via javascript.
		var tbody = document.createElement( 'tbody' );
		table.appendChild( tbody );
		
		this.processParameterList( this.dataModel.list, tbody, readOnly );
    
        //fix bug with dialog scrollbars not showing by adding extra
        //html elements to form DOM node
        for ( var i = 1; i < 2; i++ )
        {
          form.domNode.appendChild( dojo.doc.createElement( 'BR' ) );
        }
        
		this.initializeCascadingParameters();
	},

	/*****************************************************************
	* Initialize Cascading Managers
	*****************************************************************/	
	initializeCascadingParameters: function()
	{
		for ( var i in this.CascadingGroupManagers )
		{
			this.CascadingGroupManagers[i].init();
		}		
	},
	
	/*****************************************************************
	* Cleanup all cascading managers
	*****************************************************************/
	destroy: function()
	{
		for ( var i in this.CascadingGroupManagers )
		{
			this.CascadingGroupManagers[i].destroy();
		}		
	
		delete this.CascadingGroupManagers;
		
		this.CascadingGroupManagers = [];
		
        for ( i in this.ALL_WIDGETS )
        {
          this.ALL_WIDGETS[i].destroy();
        }   
        
        delete this.ALL_WIDGETS;
    
        this.ALL_WIDGETS = [];        
	},

	/*****************************************************************
	* Recursive function to build parameter table
	*****************************************************************/
	processParameterList: function ( list, tbody, form, readOnly )
	{
		//initialize variables for function
		var param  = null;
		var label  = null;
		var text   = null;
		var tr     = null;
		var td     = null;
		var widget = null;
		
		for ( var i in list )
		{
			if ( list[i].type == 2 )
			{
				tr = document.createElement( 'tr' );
				td = document.createElement( 'td' );
				
				td.style.paddingTop    = "12px";
				td.style.paddingBottom = "4px";
				td.style.fontWeight    = "bold";
				td.style.borderBottom  = "1px solid black";
				
				tr.appendChild( td );
				
				tbody.appendChild( tr );
				
				label = document.createElement( 'label' );
				label.title     = list[i].helpText;				
				
				text  = document.createTextNode( list[i].displayText ); 
				
				label.appendChild( text );
				
				td.appendChild( label );

				this.processParameterList( list[i].list, tbody, form, readOnly );
			}
			else
			{				
				param  = list[i];
				widget = null;
				
				tr = document.createElement( 'tr' );
				td = document.createElement( 'td' );

				tr.appendChild( td );
				tbody.appendChild( tr );
								
				if ( param.hidden === true )
				{
					this.createHiddenParameter( param, form );
				}
				else if ( param.selectionListType == 102 ||
						  param.selectionListType == 103 )
				{
					widget = this.createSelectListParameter( param, readOnly, td );
				}
				else if ( param.datatype == this.TYPE_STRING )
				{
					widget = this.createStringParameter( param, readOnly, td );
				}
				else if ( param.datatype == this.TYPE_FLOAT ||
						  param.datatype == this.TYPE_DECIMAL )
				{
					widget = this.createFloatParameter( param, readOnly, td );
				}
				else if ( param.datatype == this.TYPE_BOOLEAN )
				{
					widget = this.createBooleanParameter( param, readOnly, td );
				}
				else if ( param.datatype == this.TYPE_INTEGER )
				{
					widget = this.createIntegerParameter( param, readOnly, td );
				}
				else if ( param.datatype == this.TYPE_DATETIME )
				{
					widget = this.createDateTimeParameter( param, readOnly, td );
				}
				else if ( param.datatype == this.TYPE_DATE )
				{
					widget = this.createDateParameter( param, readOnly, td );
				}
				else if ( param.datatype == this.TYPE_TIME )
				{
					widget = this.createTimeParameter( param, readOnly, td );
				}
				
				if ( widget )
				{
					this.ALL_WIDGETS[i] = widget;
          
                    if ( widget.validate ) 
                    { 
                      widget.validate( false );
                    }				
                    else if ( widget.getEditor().validate ) 
                    { 
                      widget.getEditor()._hasBeenBlurred = true;
                      widget.getEditor().validate( false );
                    }       
                    
				}
			}
		}
	},
	
	/*****************************************************************
	* Get the NLS label for this parameter
	*****************************************************************/
	getLabelForParam: function ( paramInfo )
	{
		//If the parameter is required add an asterisk to the label
		var label = paramInfo.displayText;	
    				
		if ( paramInfo.isRequired === true )
		{
			label = "*" + label;
		}
		
		return label;	
	},
	
	///////////////////////////////////////////////////////////////////
	// Create HIDDEN Parameter
	///////////////////////////////////////////////////////////////////				
	createHiddenParameter: function ( paramInfo, form )
	{
		var input   = dojo.doc.createElement( 'input' );
		input.type  = "hidden";
		input.value = paramInfo.defaultValue;
		input.name  = paramInfo.name;
		
		//form.domNode.appendChild( input );
	},

	///////////////////////////////////////////////////////////////////
	// Create SELECT LIST Parameter
	///////////////////////////////////////////////////////////////////		
	createSelectListParameter: function( paramInfo, readOnly, td /*TABLE CELL*/ )
	{
		var div = dojo.doc.createElement( 'div' );
		td.appendChild( div );

        // create parameter MAP
        var map       = this._getParameterPropertyMap( paramInfo, readOnly );        
		
        // we get an object[] where each object has "key" (actual value) 
        // and "value" (UI represemtation) fields
		var kwValues = paramInfo.selectionList;
    	        
        if ( paramInfo.allowNull === true && paramInfo.isMultipleSelection !== true )
        {
          kwValues.push( { value: "_TCR_NULL", name: this.global.nls( "SELECTION_LIST_NULL_VALUE" ) } );
        }

        map.store   = new dojo.data.ItemFileReadStore( { data: { identifier: "value", items: kwValues } } );
	        
        var widget  = new ibm.tivoli.tcr.dijit.PFieldSelectionList( map, div );
        								   				        
        var manager = null;
        var params  = null;
        
        //Check to see if Cascading Parameter
        if( paramInfo.parentGroupCascading === true )
        {
            params = {  global: this.global,
                        context: this.context,
                        groupName: paramInfo.parentGroupName,
                        topic: this.topic
                     };
            
        	manager = ibm.tivoli.tcr.CascadingGroupManager.getGroupManager( params );
          
			manager.addParameter( widget, paramInfo.groupIndex );           	                                          
			
			this.CascadingGroupManagers[ paramInfo.parentGroupName ] = manager;
        }
        
        return widget;            
	},		
	
	///////////////////////////////////////////////////////////////////
	// Create STRING Parameter
	///////////////////////////////////////////////////////////////////		
	createStringParameter: function( paramInfo, readOnly, td /*TABLE CELL*/ )
	{
        var div    = dojo.doc.createElement( 'div' );
		
        td.appendChild( div );
		
        var map    = this._getParameterPropertyMap( paramInfo, readOnly );
        
		var widget = new ibm.tivoli.tcr.dijit.PFieldString( map, div );

        return widget;            
	},
	
	///////////////////////////////////////////////////////////////////
	// Create FLOAT or DECIMAL parameter
	///////////////////////////////////////////////////////////////////			
	createFloatParameter: function ( paramInfo, readOnly, td )
	{
	    var div       = dojo.doc.createElement( 'div' );
		td.appendChild( div );

        var map       = this._getParameterPropertyMap( paramInfo, readOnly );
        map.isInteger = false;

		var widget    = new ibm.tivoli.tcr.dijit.PFieldNumber( map, div );
		
        return widget;            
	},
	
	///////////////////////////////////////////////////////////////////
	// Create BOOLEAN parameter
	///////////////////////////////////////////////////////////////////
	createBooleanParameter: function ( paramInfo, readOnly, td )
	{
	    var div    = dojo.doc.createElement( 'div' );      
		td.appendChild( div );

        var map    = this._getParameterPropertyMap( paramInfo, readOnly );
		var widget = new ibm.tivoli.tcr.dijit.PFieldBoolean( map, div );

        return widget;            
	},
	
	//////////////////////////////////////////////////////////////////
	// Create INTEGER Parmater
	///////////////////////////////////////////////////////////////////		
	createIntegerParameter: function ( paramInfo, readOnly, td )
	{
	    var div    = dojo.doc.createElement( 'div' );
		td.appendChild( div );

        var map    = this._getParameterPropertyMap( paramInfo, readOnly );
		var widget = new ibm.tivoli.tcr.dijit.PFieldNumber( map, div );

        return widget;            
	},
	
	///////////////////////////////////////////////////////////////////
	// Create DATE and TIME Parameter
	///////////////////////////////////////////////////////////////////		
	createDateTimeParameter: function ( paramInfo, readOnly, td )
	{
		var div    = dojo.doc.createElement('div');			
		td.appendChild( div );

        var map    = this._getParameterPropertyMap( paramInfo, readOnly );
		var widget = new ibm.tivoli.tcr.dijit.PFieldDateTime( map, div );

        return widget;            
	},

	///////////////////////////////////////////////////////////////////
	// create DATE Parameter
	///////////////////////////////////////////////////////////////////		
	createDateParameter: function ( paramInfo, readOnly, td )
	{
	    var div    = document.createElement( 'div' );
		td.appendChild( div );
    
	    var map    = this._getParameterPropertyMap( paramInfo, readOnly );  
		var widget = new ibm.tivoli.tcr.dijit.PFieldDate( map, div );
		
        return widget;            
	},

	///////////////////////////////////////////////////////////////////
	// create TIME parameter
	///////////////////////////////////////////////////////////////////		
	createTimeParameter: function ( paramInfo, readOnly, td )
	{
		var div    = dojo.doc.createElement( 'div' );
		td.appendChild( div );
    
        var map    = this._getParameterPropertyMap( paramInfo, readOnly );    		
		var widget = new ibm.tivoli.tcr.dijit.PFieldTime( map, div );
		
        return widget;                            
	},
  
    /***************************************************************************
    * Get paramter property mapping for Widget creation
    ***************************************************************************/
    _getParameterPropertyMap: function( paramInfo, readOnly )
    {
      var map = {
                    id:             this.global.namespaceID( paramInfo.name ),
                    name:           paramInfo.name, 
                    defaultValue:   paramInfo.defaultValue,
                    required:       paramInfo.isRequired,
                    allowBlank:     paramInfo.allowBlank,
                    readonly:       readOnly,
                    helpText:       paramInfo.helpText,
                    label:          this.getLabelForParam( paramInfo ),
                    allowNull:      paramInfo.allowNull,
                    concealEntry:   paramInfo.concealEntry,                    
                    allowNewValues: paramInfo.allowNewValues,
                    isMultipleSelection: paramInfo.isMultipleSelection
                };
                
      return map;                
    }			
});

dojo.provide( "ibm.tivoli.tcr.CascadingGroupManager" );
ibm.tivoli.tcr.CascadingGroupManager.instances = [];
/****************************************************************************
* CascadingGroupManager -- singleton manager
*
* required context objects:
* global: global tcr object
* groupName: unique group name
****************************************************************************/
ibm.tivoli.tcr.CascadingGroupManager.getGroupManager = function( context )
{
  var groupNameID = context.global.namespaceID( context.groupName );
  
  var singleton = ibm.tivoli.tcr.CascadingGroupManager.instances[ groupNameID ];
  
  //if a singleton was not found then assume we need to create one now0
  if ( singleton === undefined )
  {    
    singleton = new ibm.tivoli.tcr._CascadingGroupManager( context );
   
    //save the new singleton instance on the Global memory space
    ibm.tivoli.tcr.CascadingGroupManager.instances[ groupNameID ] = singleton;
  }  
  else
  	singleton.context = context.context;
  return( singleton );  
};

/****************************************************************************
* _CascadingGroupManager -- the actual CascadingGroupManager object
*
* Notice that there is a '_' underscore prefixing this object.  This is
* important because removing it will delete the CascadingGroupManager module 
* functions such as getSingleton.  This is OK because we will never construct 
* one of these things without calling getSingleton.
****************************************************************************/
dojo.declare("ibm.tivoli.tcr._CascadingGroupManager",null,
{      
  constructor: function( context )
  {
      //add all functions, fields, objects inside context to this object
      dojo.mixin( this, context );

      this._values = [];
      this._listeners = [];      
      this._updateInProgress = false;
  },
  
  /********************************************************
  * Add a parameter widget to this manager
  ********************************************************/
  addParameter: function( widget, groupId )
  {
     //console.debug( "widget", widget );

     var widgetInfo =
     {
       id   : widget.id,
       value: ( widget.defaultValue ) ? widget.defaultValue : null      
     };
     
     //console.debug( "widgetInfo", widgetInfo );
     
     this._values[ groupId ] = widgetInfo;
     
     this._listeners [ groupId ] = 
  		new ibm.tivoli.tcr.CascadingParamListener( this, groupId, widget );
  },
  
  /********************************************************
  * Each param change listener will call this method
  ********************************************************/
  onValueChanged: function( groupId, value )
  {		
    if ( ! this._updateInProgress )
    {
      if ( value !== undefined )
      {
        this._values[ groupId ].value = value;
      
        var isLastInGroup = this._values.length == ( groupId + 1 );
    
        if ( !isLastInGroup )
        {
          this.requestGroupUpdate( groupId );
        }
      }
    }
  },
  
  /********************************************************
  * return all parameter values as array up to given
  * groupId
  ********************************************************/
  getParameters: function( groupId )
  {
  	var params = [];
  	    
  	for ( var i = 0; i <= groupId; i++ )
  	{
  		params[i] = this._values[i].value;
  	}
      	
  	return params;
  },
  
  /********************************************************
  * Given the groupId that just changed request updates
  ********************************************************/
  requestGroupUpdate: function( groupId )
  {
    this._updateInProgress = true;
  	kw = { 
           content:    {
                         topic      : this.topic,
                         command    : "getCascadedParameters",   
                         context    : this.context,
                         parameters : dojo.toJson( { groupName : this.groupName,    
                                                     cascades  : this.getParameters( groupId ) } )
                       },
           
           groupId:    groupId,  // cache for use in "onResponse" handler
           
           onResponse: { callback: dojo.hitch( this, "_responseGroupUpdate" ) },
           onError: { callback: dojo.hitch( this, function(){ this._updateInProgress = false; } ) }
         };
  
      ibm.tivoli.tcr.Server.getSingleton( this.global ).get( kw );
  },

  _responseGroupUpdate: function( data, kw ) 
  {
     this.populateGroup( data, kw.groupId );
     this._updateInProgress = false;
  }, 
  
  /********************************************************
  * Given the data response from an update -- populate
  * the group with new values
  *
  * This method does not run in manager scope.  It is
  * called from a dojo.io.bind -- and therefore must have
  * a reference to this manager.
  ********************************************************/
  populateGroup: function( data, groupId )
  {
  	var params = [];
  	var param  = null;
    var widget = null;
  	
  	//build a list of parameters returned in groupIndex order
  	//this may not actually be required because they already
  	//may be in order -- but this is defensive coding
  	for ( var i in data.list )
  	{
  		param = data.list[i];
  		
  		params[ param.groupIndex ] = param;
  	}
  	
  	for ( i in params )
  	{
      widget = dijit.byId( this._values[ params[i].groupIndex ].id );

      this.populateComboBoxWithValues( params[i].selectionList,
                                       params[i].defaultValue,
                                       widget,
                                       params[i].allowNull );
      
      if ( params[i].selectionList !== null )
      {
      	widget.getEditor().setDisabled( false );
      }
  	}
  },
  
  /********************************************************
  * Given a combobox widget, a list of values from JSON, 
  * and a default value -- it populates a combobox with 
  * values.
  ********************************************************/
  populateComboBoxWithValues: function ( values, defaultValue, paramWidget, allowNull )
  {	     
    if ( allowNull === true )
    {
      values.push( { value: "_TCR_NULL", name: this.global.nls( "SELECTION_LIST_NULL_VALUE" ) } );
    }
                
    paramWidget.getEditor().setStore( new dojo.data.ItemFileReadStore( { data: { identifier: "value", items: values } } ) );
    
    paramWidget.getEditor().setValue( defaultValue );
    
    //after setting new value on combobox -- get it to validate and adjust visibility of errors
    paramWidget.getEditor()._hasBeenBlurred = true;
    paramWidget.getEditor().validate( false );
            
  },
  
  /********************************************************
  * This method is called after all params have been added
  ********************************************************/
  init: function()
  {
  	var parentIsDisabled = false;
  	var param = null;
    var value = null;
  	var lastGroupIndex = 0; /*last value valid parameter found*/
  	
  	for ( var i in this._values )
  	{
  		if ( i < 1 ) { continue; } //first param is always enabled
  					  		       //and contains potential values
  		  		
  		//get the current value of the parameter (from HTML form)
  		value = this._values[i].value;
        param = dijit.byId( this._values[i].id );
  					
  		//param.isValid() doesn't work at this point
  		//need to find out why
  		
  		var paramInvalid = ( (value === ""   && !param.allowBlank)   ||
  							 (value === null && !param.allowNull) );
  		
  		if ( parentIsDisabled === true || paramInvalid )
  		{
  			param.getEditor().setDisabled( true );
  			
  			//set a flag that a field has been disabled
  			//so all children of the parameter are disabled
  			//too
  			parentIsDisabled = true;
  		}
  		else
  		{
  			//record the last group index that contained a valid
  			//value -- this will be used to update the entire
  			//parameter group with values from the server
  			lastGroupIndex =  i;
  		}						
  	}
  	
  	this.requestGroupUpdate( lastGroupIndex );		
  },
  
  /********************************************************
  * Destroy this object -- remove all listeners
  ********************************************************/
  destroy: function()
  {
  	delete this._values;
    
    //destroy all event listeners
    for ( var i in this._listeners )
    {
      this._listeners[i].destroy();
    }
    
    delete this._listeners;
  	
  	//you MUST re-initialize these variables
  	//or subsequent uses of the manager will fail
  	this._values = [];
    this._listeners = [];
  }	
});

////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Cascading Parameters Listener
//
// The purpose of this listener is to maintain a reference to the
// parameter widget being listened to as well as the groupIndex.  
// Using dojo.event.connect does NOT return a reference to the 
// widget that it is attached to
////////////////////////////////////////////////////////////////////////////////////////////////////////////////
dojo.declare( "ibm.tivoli.tcr.CascadingParamListener", //className
        null,                                          //extends
{
  //acts like a java constructor
  constructor: function( group, groupId, widget )
  {
    this.group    = group;    
    this.groupId  = groupId;    
    this.editor   = widget.getEditor();     

    this.connection = dojo.connect( this.editor, "onChange", this, "onValueChanged" );
  },
  
  onValueChanged: function( value )
  {
    //console.log( "combobox changed!!!!!!!!!!!!" + value );
    //tell the group manager that a parameter has changed its
    //value -- provide the groupIndex of the parameter that
    //changed
    this.group.onValueChanged( this.groupId, value );
  },
  
  destroy: function ()
  {
    if ( this.connection ) dojo.disconnect( this.connection );
    delete this.connection;
  }  
});
