/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
/*******************************************************************************
 * CMVC Location: %W%
 * Version:       %I%
 * Last Modified: %G%
 *******************************************************************************/

dojo.provide("ibm.tivoli.tip.charts.dijit.ConnectionManagerWidget");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Toolbar");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.ValidationTextBox");
dojo.require("ibm.tivoli.tip.dijit.TIPTable");
dojo.require("ibm.tivoli.tip.dijit.TIPMessageDialog");
dojo.require("ibm.tivoli.tip.dijit.TIPContextMenuV2");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("ibm.tivoli.tip.dojo.Global");
dojo.require("ibm.tivoli.tip.dojo.Server");

// NLS
dojo.requireLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");
dojo.requireLocalization("ibm.tivoli.tip.charts.dijit", "chart_messages");


// define the widget class
dojo.declare(
	"ibm.tivoli.tip.charts.dijit.ConnectionManagerWidget", 
    [ dijit._Widget, dijit._Templated ],
{
    divToChange_ : "connectionManagerDiv",
    namespace_ : "namespace1_",
    iscPortletNamespace : null,
    callback_ : null,
    global: null,
  	// widgetsInTemplate Boolean:
	//		should we parse the template to find widgets that might be
	//		declared in markup inside it? false by default.
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("ibm.tivoli.tip.charts.dijit","templates/ConnectionManagerWidget.html"),

	constructor: function() {
	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");
	    this.messages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_messages");
	    
	    this.connections = [];
	},
	initialize: function() { 
	},
		
	postCreate: function() {
		this.inherited ("postCreate", arguments);
		this.global = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace_);
		var createButton = dojo.byId (this.id + "createBtn");
        this.connections.push (dojo.connect(createButton, 'onclick', this, 'onCreateClick'));

		var cancelButton = dojo.byId (this.id + "cancelBtn");
        this.connections.push (dojo.connect(cancelButton, 'onclick', this, 'onCancelClick'));

        this.createConnectionsTable ();
	},
	onCreateClick: function() {
		var name = dojo.byId(this.id + "name").value;
//		var protocol = dojo.byId(this.id + "protocol").value;
		var protocolSelect = dojo.byId (this.id + "protocol"); 
		var protocol = protocolSelect.options[protocolSelect.selectedIndex].value; 
		
		var hostname = dojo.byId(this.id + "hostname").value;
    var portField = dijit.byId(this.id + "port");
		var port = portField.getValue();
		var serviceName = dojo.byId(this.id + "service_name").value;
		// Remove for now, not used as all apps within TIP should be SSO
		// ITM is only exception and usr/pwd handled by installer.
		// May be introduced at later point.
		//var username = dojo.byId(this.id + "username").value;
		//var password = dojo.byId(this.id + "password").value;		
		var username = "";
		var password = "";
	console.debug("--isInrange="+portField.isInRange()+" isValid="+portField.isValid()+" port="+port);	
		if ((!name) || (!protocol) || (!hostname) || (!serviceName)) {
			this.showRequiredFieldsDialog();
    } else if ( !isNaN(port) && !portField.isInRange() ) {
      this.showFieldValueOutOfRangeDialog(); 
    } else if ( !portField.isValid() ) {
      this.showInvalidFieldValueDialog(); 
		} else {
			var kw = {
	             content:   {
                        command: "saveConnection",
						arg_name: encodeURI(name),
						arg_protocol: protocol,
						arg_hostname: encodeURI(hostname),
						arg_port: port,
						arg_service_name: encodeURI(serviceName),
						arg_username: encodeURI(username),
						arg_password: password
                        },
            	 onResponse:{
                         callback: dojo.hitch(this, this.updateConnectionsTable)
       	                }
    	     };
	
	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		
			this.showListConnections();
//			dojo.publish("/com.ibm.tivoli.tip.charts", [ "saveConnection" ]);
		}
	},
	showRequiredFieldsDialog: function () {
		var helper = new ibm.tivoli.tip.dojo.TIPHelper();
        var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: "error",
		                                              portletTitle: helper.getPortletTitle(this.domNode),
	    	                                          message: this.messages_.TIPCH3006E,
	        	                                      messageId: "TIPCH3006E"
	                    	                        }); 
		modalWindow.show();
	},
  showInvalidFieldValueDialog: function () {
		var helper = new ibm.tivoli.tip.dojo.TIPHelper();
    var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: "error",
		                                              portletTitle: helper.getPortletTitle(this.domNode),
	    	                                          message: this.messages_.TIPCH3009E,
	        	                                      messageId: "TIPCH3009E"
	                    	                        }); 
		modalWindow.show();
  },
  showFieldValueOutOfRangeDialog: function () {
		var helper = new ibm.tivoli.tip.dojo.TIPHelper();
    var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: "error",
		                                              portletTitle: helper.getPortletTitle(this.domNode),
	    	                                          message: this.messages_.TIPCH3010E,
	        	                                      messageId: "TIPCH3010E"
	                    	                        }); 
		modalWindow.show();
  },
	onCancelClick: function () {
		dojo.byId(this.id + "name").value = "";
		var protocolSelect = dojo.byId (this.id + "protocol"); 
		protocolSelect.selectedIndex = 0;
//		dojo.byId(this.id + "protocol").value = "";
		dojo.byId(this.id + "hostname").value = "";
		dojo.byId(this.id + "port").value = "";
		dojo.byId(this.id + "service_name").value = "";
		//dojo.byId(this.id + "username").value = "";
		//dojo.byId(this.id + "password").value = "";		

		this.showListConnections();
//		dojo.publish("/com.ibm.tivoli.tip.charts", [ "cancelConnection" ]);
	},
	newConnection: function () {
//		var listConns = dojo.byId (this.id + "listConnectionsDiv");
//		listConns.style.display = 'none';
//		var newConn = dojo.byId (this.id + "connectionProperties");
//		newConn.style.display = '';
		this.dialog.show();
		
//		dojo.publish("/com.ibm.tivoli.tip.charts", [ "newConnection" ]);
	},
	deleteConnection: function () {
		var connectionName = this.getSelectedDataSourceName();

		if (connectionName) {
			var kw = {
	             content:   {
                        command: "deleteConnection",
						arg_connectionName: encodeURI(connectionName)
                        },
            	 onResponse:{
                         callback: dojo.hitch(this, this.updateConnectionsTable)
       	                }
    	     };
	
	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		} else {
			var helper = new ibm.tivoli.tip.dojo.TIPHelper();
    	    var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: "error",
													  portletTitle: helper.getPortletTitle(this.domNode),
	    	                                          message: this.messages_.TIPCH3007E,
	        	                                      messageId: "TIPCH3007E"
	                    	                        }); 
			modalWindow.show();
		}
	},
	showListConnections: function () {
//		var listConns = dojo.byId (this.id + "listConnectionsDiv");
//		listConns.style.display = '';
//		var newConn = dojo.byId (this.id + "connectionProperties");
//		newConn.style.display = 'none';
		this.dialog.hide();
	},
	selectConnection: function (connectionName) {
		console.info ("enter selectionConnection with name: " + connectionName);
		var tbl = dijit.byId (this.id + "tbl_listConnections");
		console.info ("got tbl: ");
		console.info (tbl);
		var row = null;

		loop:
		for (var rowIndex=0; rowIndex < tbl.getModel().getRowCount(); rowIndex++) {
			console.info ("checking row: " + rowIndex);
			var tblConnectionName = tbl.getValueAt (rowIndex, 0);
			if (tblConnectionName == connectionName) {
		    	console.info ("found connectionName in table... selecting row: " + rowIndex);
		    	tbl.getGrid().selection.select (rowIndex);
		    	break loop;
		    }
		}
		
		console.info ("exit selectConnection");
	},
	getSelectedDataSourceName: function () {
		var tbl = dijit.byId (this.id + "tbl_listConnections");
		var selectedRow = tbl.getSelectedRowIndexes();
		console.log ("ConnectionManagerWidget.getSelectedDataSourceName. selectedRows: " + selectedRow);
		if ((selectedRow) && (selectedRow.length > 0)) {
			var connectionName = tbl.getValueAt (selectedRow, 0);
			console.log ("ConnectionManagerWidget.getSelectedDataSourceName.  connectionName: " + connectionName);
			return (connectionName);
		} else {
			console.log ("ConnectionManagerWidget.getSelectedDataSourceName: no row selected");
			return (null);
		}
	},
	//  Update the filtering table identified by view with fresh data from the backend.
	createConnectionsTable: function () {
		var global = ibm.tivoli.tip.dojo.Global.getSingleton (this.namespace_);
		
		var cmProvider = new ibm.tivoli.tip.dijit.CMSMenuProviderV2(
				{ global: global,
				  menuName: "listConnectionsMenu",
				  parentid: this.id,
				  hasLocalActions: false,
				  iscPortletNamespace: this.iscPortletNamespace
				});
	
		var tipTable = new ibm.tivoli.tip.dijit.TIPTable({
					global: global,
					tableID: "tbl_listConnections",
					multiSelect: false,
					cmProvider: cmProvider,
					onModelAllChange: this.callback_,
					width: "100%",
					gridParms: {autoWidth: false, autoHeight:false, rowsPerPage: "10", rowCount: "5" },
					id: this.id + "tbl_listConnections"
				}, 
				dojo.byId (this.id + "tbl_listConnections"));
				
		tipTable.startup();
//		var footerToolbar = tipTable.getFooterToolbar().getToolbar();
		var footerToolbar = tipTable.getFooterToolbar();

		var back = new ibm.tivoli.tip.dijit.TIPButton ({
							label: this.resources_.BACK,
							showLabel: true,
							onClick: dojo.hitch (this, this.onBack)
						});
		footerToolbar.addChild (back);
		
		var next = new ibm.tivoli.tip.dijit.TIPButton ({
							label: this.resources_.NEXT,
							showLabel: true,
							onClick: dojo.hitch (this, this.onNext)
						});
		footerToolbar.addChild (next);
		
		
		var context = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace_);
		console.log ("got context:");
		console.log (context);
		if ( context.roles.admin ) {
			console.log ("user is a chartAdministrator. constructing new/delete connections buttons");
			var toolbar = tipTable.getToolbar();

			var button = new ibm.tivoli.tip.dijit.TIPButton(
			{	
				iconClass: "newIcon",
				label: this.resources_.NEW,
				showLabel: false,
			 	onClick: dojo.hitch (this, this.newConnection) 
			});
			toolbar.addChild(button);
		
			button = new ibm.tivoli.tip.dijit.TIPButton(
			{	
				iconClass: "deleteIcon",
				label: this.resources_.DELETE,
				showLabel: false,
			 	onClick: dojo.hitch (this, this.deleteConnection) 
			});
			toolbar.addChild(button);
		} else {
			console.log ("user is not a chartAdministrator");
		}
		
		dojo.subscribe (tipTable.id + "#LoadComplete", this, "onTableLoad");
		this.resize();
	},
	onTableLoad: function () {
		// We have to do this to accommodate a bug in TIPTable where toolbars and footers 
		// don't appear until after the data is loaded and resize is called again
		console.log ("enter ConnectionManagerWidget.onTableLoad");
		this.resize();
		
		// We have to refresh to data again to accommodate a bug in TIPTable selection
		dijit.byId (this.id + "tbl_listConnections").onRefresh();
		console.log ("exit ConnectionManagerWidget.onTableLoad");
	},
	resize: function () {
		var portletSize = iscPortlet.getPortletSize(this.global.iscPortletNamespace);
		var size = { w: portletSize.available.w, h: portletSize.available.h };
		if (size.w === undefined) {
			// guess
			size.w = 850;
		} else if (isNaN(size.w)) {
			// guess
			size.w = portletSize.current.w;
		} 
		if (size.h === undefined) {
			// guess
			size.h = 200;
		} else if (isNaN(size.h)) {
			// guess
			size.h = 200;
		} 
		
		console.log ("ConnectionManagerWidget.resize: portlet size: ");
		console.log (portletSize);
		console.log ("size: ");
		console.log (size);
		//dijit.byId (this.namespace_ + "mainChartNavigatorDiv").resize(size);
		dijit.byId (this.id + "tbl_listConnections").resize (size);
	},
	onBack: function () {
		dijit.byId (this.namespace_ + "mainChartNavigatorDiv").showChartNavigator();
	},
	onNext: function () {
		dijit.byId (this.namespace_ + "mainChartNavigatorDiv").showCMChartSelector();
	},
	updateConnectionsTable: function (data, ioArgs) {
		var tbl = dijit.byId (this.id + "tbl_listConnections");
		tbl.onRefresh();
	},
	destroy: function () {
		console.log ("enter ConnectionManagerWidget.destroy");
	
		dojo.forEach (this.connections, function (item) {
			dojo.disconnect (item);
		});
		this.dialog.destroy();
		
//		this.destroyDescendants();
		this.inherited ("destroy", arguments);
		console.log ("exit ConnectionManagerWidget.destroy");
	}
});
